//+------------------------------------------------------------------+
//|                                          WriteAllChartsToCSV.mq4 |
//|                                                               me |
//|                                                                  |
//+------------------------------------------------------------------+
//
//As the name implies, this indicator writes the bar chart of all available symbols' 5m,15m,1h,1d,1w to .csv files.
//It also, updates the data in (almost) realtime. [NOT YET]

#property copyright "me"
#property link      ""

#property indicator_chart_window


extern int BarHistory=2000;


string FileNames[6];  // 5m ,15m, 1h, 1d, 1w
int Periods[6] = { PERIOD_M1, PERIOD_M5, PERIOD_M15, PERIOD_H1, PERIOD_D1, PERIOD_W1}; 
string FileNameEnds[6] = {"1m.csv", "5m.csv","15m.csv","1h.csv","1d.csv","1w.csv"};

int TotalSymbols;
string SymbolArray[];

/*
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
	TotalSymbols = StoreSymbols(SymbolArray);

	for(int i =0; i< TotalSymbols; i++){


		for(int PeriodIndex=0; PeriodIndex<ArraySize(Periods) ; PeriodIndex++ ){

			FileNames[PeriodIndex] = StringSubstr(SymbolArray[i],0,6) + FileNameEnds[PeriodIndex] ;      //We are interested in 5 periods for each symbol

			FileDelete(FileNames[PeriodIndex]);    //makes a new files.  
			int Handle  = FileOpen(FileNames[PeriodIndex] , FILE_CSV|FILE_READ|FILE_WRITE, "\t");

			if (Handle<1){
				Print("Error opening file in init(): ",GetLastError());
				return(0);
			}


			string Strline;
			for (int BarIndex=BarHistory; BarIndex>0; BarIndex--){  //last bar not handled here

				Strline = MakeInfoString(SymbolArray[i],Periods[PeriodIndex] ,BarIndex);
				FileWrite(Handle, Strline);

			}   //we wrote 2000 lines in the file, 2000 to 1 inclusive


			FileClose(Handle);

		}

	} //now next symbol


}   //At this point all the files should be filled with all data except last bar	

//+------------------------------------------------------------------+
*/


//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{

}



//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
{
	TotalSymbols = StoreSymbols(SymbolArray);   

	for(int i =0; i< TotalSymbols; i++){			//for each symbol


		for(int PeriodIndex=0; PeriodIndex< ArraySize(Periods) ; PeriodIndex++ ){		 //We are interested in 5 periods for each symbol

			FileNames[PeriodIndex] = StringSubstr(SymbolArray[i],0,6) + FileNameEnds[PeriodIndex] ;
			
			FileDelete(FileNames[PeriodIndex]);    //JUST FOR TESTING !!!!             
			
			
			int Handle  = FileOpen(FileNames[PeriodIndex] , FILE_CSV|FILE_READ|FILE_WRITE, "\t");
			if (Handle<1){
				Print("Error opening file in start(): ",GetLastError());
				return(0);
			}

			//get last string data
			string LastTickString = GetLastLine(Handle);
			datetime TimeStamp = StrToTime(StringSubstr(LastTickString,0,4) + "." + StringSubstr(LastTickString,5,2) + "." 
				+ StringSubstr(LastTickString,8,2) + " " + StringSubstr(LastTickString,11,5)   ) ;               
			double LastClosePrice = NormalizeDouble(   StrToDouble( ExtractString(LastTickString, 44 , 6) ), 4 ); //44 for comma, 6 is the end comma

//			Print( SymbolArray[i]+Periods[PeriodIndex] + ": TimeStamp= " + TimeStamp + ", LastClosePrice= " + LastClosePrice);


			//----------Check for new bar -----------||
			if(TimeStamp != iTime(SymbolArray[i],Periods[PeriodIndex], 0) ){    //this conditon should be met on the first start() iteration

				TimeStamp = iTime(SymbolArray[i],Periods[PeriodIndex], 0);

				LastTickString = MakeInfoString(SymbolArray[i],Periods[PeriodIndex] ,0);
				FileSeek(Handle,0, SEEK_END); 
				FileWrite(Handle, LastTickString);				
			}	

			//----------Check for different close-----------||
			else if(LastClosePrice != iClose(SymbolArray[i],Periods[PeriodIndex] ,0) ){   //if not a newbar, but a new tick, we should delete the last line and rewrite		

				int ByteCount = StringLen(LastTickString) + 2 ; //count number of bytes to go back

				LastTickString = MakeInfoString(SymbolArray[i],Periods[PeriodIndex] ,0);
				FileSeek(Handle,-1*ByteCount, SEEK_END);
				FileWrite(Handle, LastTickString);
			}

			FileClose(Handle);


		} // next period



	}  //next symbol



	return(0);

}
//+------------------------------------------------------------------+





string MakeInfoString(string symbol, int period, int Bar){

	string Strline = TimeToStr(iTime(symbol,period, Bar),TIME_DATE|TIME_SECONDS);

	Strline = StringSubstr(Strline,0,4) + StringSubstr(Strline,5,2) + StringSubstr(Strline,8,2) + "," + StringSubstr(Strline,11,5);

	Strline = Strline + "," + DoubleToStr(iOpen(symbol,period, Bar),4) + "," + DoubleToStr(iHigh(symbol,period, Bar),4) + "," 
		+ DoubleToStr(iLow(symbol,period, Bar),4) + "," + DoubleToStr(iClose(symbol,period, Bar),4) + "," + DoubleToStr(iVolume(symbol,period, Bar),0);

	//for 5 digit brokers, you'll nedd to adjust the 4s. Do it later. use  MarketInfo(Currency,MODE_DIGITS)

	return(Strline);
}



//Stores the symbols into the givin array and returns the number of symbols
int StoreSymbols(string& SymbolArray[]) 
{
	int handle=FileOpenHistory("symbols.raw", FILE_BIN | FILE_READ);
	if(handle<1)
	{
		Print("Error Opening File in StoreSymbols(): ", GetLastError());
		return(false);
	}  

	int TotalRecords=FileSize(handle) / 1936;
	ArrayResize(SymbolArray, TotalRecords);

	string SymbolName;
	for(int i=0; i<TotalRecords; i++) 
	{
		SymbolName =FileReadString(handle, 12);
		FileSeek(handle, 1924, SEEK_CUR); // goto the next record

		SymbolArray[i]=SymbolName;

	}

	FileClose(handle);

	return(TotalRecords);
}



//Returns the last line in a file. Exits with the pointer position at where it was.
string GetLastLine(int Handle){

	int initpos = FileTell(Handle);
	string Read = "";
	int ByteCount=3;

	while( True){

		FileSeek(Handle,-1*ByteCount, SEEK_END);  //at first loop it is the last char in the last line
		Read = FileReadString(Handle);


		if ( Read == ""){  //we hit the "\r\n"

			//roll back 
			ByteCount--;      
			FileSeek(Handle,-1*ByteCount, SEEK_END);  //at first loop it is the last char in the last line
			Read = FileReadString(Handle);

			return(Read);
		}	

		ByteCount++;
	}

	FileSeek(Handle,initpos, SEEK_SET);	
}



//Given a string and delimiter code (ASCII), the function extracts the string between the EndCount^th delimeter and the previous.
string ExtractString(string Str, int DelimiterCode, int EndCount){

	int pos,startpos;
	for( int CommaCount = 0; CommaCount<EndCount && pos<StringLen(Str) ; pos++){ 

		if( StringGetChar(Str,pos) == DelimiterCode) CommaCount++;  //comma is 44

		if( CommaCount == 5 && !startpos) startpos = pos + 1;  //if startpos hasn't been set

	}

	return(StringSubstr(Str,startpos, pos-startpos-1) );

}